C*********************************************************************
C*********************************************************************
C** 
C**     File: cmdln.for
C**
C**     Project: jitax77
C**
C**     Description: Routines for reading the command line.
C**                  Uses f2kcli from Interactive Software
C**                  Services Ltd.
C**                  See http://www.winteracter.com/f2kcli
C**                  The f2kcli code is used under licence and
C**                  I.S.S. Ltd. retains the copyright for f2kcli.
C**                  This does not affect the jitax77 licence.
C**
C*********************************************************************
C**
C** Copyright (c) 2003 Scott A. Belmonte
C**
C** E-mail: scott.belmonte@ntlworld.com
C**
C** This file is part of jitax77.
C**
C** jitax77 is free software; you can redistribute it and/or modify
C** it under the terms of the GNU General Public License as published
C** by the Free Software Foundation; either version 2 of the License,
C** or (at your option) any later version.
C**
C** jitax77 is distributed in the hope that it will be useful,
C** but WITHOUT ANY WARRANTY; without even the implied warranty of
C** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C** GNU General Public License for more details.
C**
C** You should have received a copy of the GNU General Public License
C** along with jitax77; if not, write to the Free Software
C** Foundation, Inc., 59 Temple Place, Suite 330, Boston,
C** MA  02111-1307  USA.
C**
C*********************************************************************
C*********************************************************************
C
C*********************************************************************
C     Routine: PARSE_OPTIONS
C
C     Description:
C        This subroutine parses the command line options. It uses 
C        routines provided by f2kcli to do this.
C*********************************************************************
      SUBROUTINE PARSE_OPTIONS()
      IMPLICIT NONE
      INCLUDE 'f2kcli.inc'
      INCLUDE 'cmdln.inc'
C
C     Variables
C
      INTEGER      NARGS          ! Number of command line arguments
      CHARACTER*40 ARG            ! Command line arguments
      INTEGER      I, LEN, STATUS
      INTEGER      SKIP           ! Number of arguments to skip
C     
C     Initialise options.
C     
      BAKFIL     = ''
      EXPNAM     = ''
      NMERGE     = 0
      DEBUG      = .FALSE.
      BIGEND_FLF = .TRUE.
      AVERAGE    = .FALSE.
      DYNBAK     = .FALSE.
      COSMIC     = 0
C     
      NARGS = COMMAND_ARGUMENT_COUNT()
      IF (NARGS .EQ. 0) CALL SHOW_OPTIONS()
C     
      SKIP = 0
      DO I = 1, NARGS
         IF (SKIP .GT. 0) THEN
            SKIP = SKIP - 1
         ELSE
            CALL GET_COMMAND_ARGUMENT(I, ARG, LEN, STATUS)
            IF ((ARG .EQ. '-e') .AND. (NARGS .GT. I+2)) THEN
C
C     Read the experiment name and start and end frame numbers
C
               CALL GET_COMMAND_ARGUMENT((I+1), ARG, LEN, STATUS)
               EXPNAM = ARG
               CALL GET_COMMAND_ARGUMENT((I+2), ARG, LEN, STATUS)
               READ(ARG, '(BN, I4)', ERR=901) START_FRAME
               CALL GET_COMMAND_ARGUMENT((I+3), ARG, LEN, STATUS)
               READ(ARG, '(BN, I4)', ERR=901) END_FRAME
               SKIP = 3
            ELSE IF ((ARG .EQ. '-b') .AND. (NARGS .GT. I)) THEN
C
C     Read the background file name
C
               CALL GET_COMMAND_ARGUMENT((I+1), ARG, LEN, STATUS)
               BAKFIL = ARG
               SKIP = 1
            ELSE IF ((ARG .EQ. '-m') .AND. (NARGS .GT. I)) THEN
C
C     Read the number of frames to merge
C
               CALL GET_COMMAND_ARGUMENT((I+1), ARG, LEN, STATUS)
               READ(ARG, '(BN, I4)') NMERGE
               SKIP = 1
            ELSE IF ((ARG .EQ. '-c') .AND. (NARGS .GT. I)) THEN
C
C     Read the number of frames to use in cosmic ray correction
C
               CALL GET_COMMAND_ARGUMENT((I+1), ARG, LEN, STATUS)
               READ(ARG, '(BN, I4)') COSMIC
               SKIP = 1
            ELSE IF (ARG .EQ. '-v') THEN
C
C     Turn verbose (debug) mode on.
C
               DEBUG = .TRUE.
            ELSE IF (ARG .EQ. '-l') THEN
C
C     The FLF output should be little endian.
C
               BIGEND_FLF = .FALSE.
            ELSE IF (ARG .EQ. '-a') THEN
C
C     Merged files should be averaged. Default is to add files.
C
               AVERAGE = .TRUE.
            ELSE IF (ARG .EQ. '-d') THEN
C
C     Calculate a dynamic background and subtract it from frames.
C
               DYNBAK = .TRUE.
            ELSE
               CALL SHOW_OPTIONS() 
            ENDIF
         ENDIF
      ENDDO
      RETURN
C
C     Error trap
C
 901  WRITE(*,*) '** Error reading frame numbers.'
      CALL SHOW_OPTIONS
      RETURN
      END
C
C
C
C*********************************************************************
C     Routine: SHOW_OPTIONS
C
C     Description:
C        Prints out the usage instructions in the case of a malformed
C        command line.
C*********************************************************************
      SUBROUTINE SHOW_OPTIONS()
      IMPLICIT NONE
      WRITE(*,*) 'jitax77 V1.2'
      WRITE(*,*) 'Usage: '
      WRITE(*,*) 
     $  '  jitax77 -e expnam # # [-b backfile] [-m #] [-a] [-l] [-v]'
      WRITE(*,*) '-e expnam # #: mandatory, expnam is the base'
      WRITE(*,*) '               experiment name. # # are the'
      WRITE(*,*) '               start and end frame numbers.'
      WRITE(*,*) '               e.g. -e nacl 100 144 processes the'
      WRITE(*,*) '               files nacl100.tif to nacl144.tif.'
      WRITE(*,*) '-b backfile: optional, backfile is the background'
      WRITE(*,*) '             TIFF file without the .tif extension.'
      WRITE(*,*) '             The background file is subtracted'
      WRITE(*,*) '             every merged FLF file.'
      WRITE(*,*) '-m #: optional, adds # frames together into a'
      WRITE(*,*) '      single FLF file.'
      WRITE(*,*) '-a: optional, when combined with the -m flag the'
      WRITE(*,*) '    merged frames are averaged instead of added.'
      WRITE(*,*) '-l: optional, writes out FLF files in little-endian'
      WRITE(*,*) '    format (the default is big-endian).'
      WRITE(*,*) '-v: optional, verbose (debug) mode. Outputs debug'
      WRITE(*,*) '    information to the terminal.'
      WRITE(*,*) '-d: optional, calculates and subtracts dynamic'
      WRITE(*,*) '    backbground. (See documentation for details.)'
      WRITE(*,*) '-c #: optional, performs cosmic ray correction in'
      WRITE(*,*) '      groups of # frames. (See documentation.)'
      STOP
      END
C
C
C
