C**********************************************************************
C**********************************************************************
C** 
C**     File: jitax77.for
C**
C**     Project: jitax77
C**
C**     Description: jitax77 reads CCD image data in TIFF format and
C**                  subtracts a background image (also in TIFF format)
C**                  and outputs the final data in an FLF format file
C**                  for each input TIFF file.
C**
C**********************************************************************
C**
C** Copyright (c) 2003 Scott A. Belmonte
C**
C** E-mail: scott.belmonte@ntlworld.com
C**
C** This file is part of jitax77.
C**
C** jitax77 is free software; you can redistribute it and/or modify
C** it under the terms of the GNU General Public License as published
C** by the Free Software Foundation; either version 2 of the License,
C** or (at your option) any later version.
C**
C** jitax77 is distributed in the hope that it will be useful,
C** but WITHOUT ANY WARRANTY; without even the implied warranty of
C** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C** GNU General Public License for more details.
C**
C** You should have received a copy of the GNU General Public License
C** along with jitax77; if not, write to the Free Software
C** Foundation, Inc., 59 Temple Place, Suite 330, Boston,
C** MA  02111-1307  USA.
C**
C**********************************************************************
C**********************************************************************
C
      PROGRAM JITAX77
      IMPLICIT NONE
C     
      INCLUDE 'cmdln.inc'
      INCLUDE 'flf.inc'         ! flf.inc must come before jitax77.inc
      INCLUDE 'jitax77.inc'
C
C     Functions
C
      INTEGER OPEN_TIFF_FILE
      INTEGER READ_TIFF_DATA
      INTEGER TIFF_IMAGE_WIDTH
      INTEGER TIFF_IMAGE_HEIGHT
      INTEGER WRITE_FLF_FILE
      INTEGER STRLEN
C
C     Variables
C
      INTEGER TIFF_ERROR
      INTEGER I,J               ! Loop counters
      INTEGER DATA(MAXIMSIZE)   ! Raw data array
      INTEGER FRAMENUM          ! Current frame number
      INTEGER BAKWIDTH          ! Background frame width
      INTEGER BAKHEIGHT         ! Background frame height
      INTEGER NSATPIX           ! Number of saturated pixels
      INTEGER NPROCESSED        ! Cosmic ray corrected frames
      CHARACTER*4  TIFFNUM      ! The current TIFF file number
      CHARACTER*88 TIFFNAME     ! The current TIFF file name
      CHARACTER*88 FLFNAME      ! The current FLF file name
      CHARACTER*4  FLFNUM       ! The current FLF file number
C     
      TIFFNUM = ''
      TIFFNAME = ''
      FLFNAME = ''
      FLFNUM = ''
      CALL PARSE_OPTIONS()
C
C     Check experiment name.
C
      IF (STRLEN(EXPNAM) .EQ. 0) THEN
         WRITE(*,*) '** Error: No experiment name'
         CALL SHOW_OPTIONS
      ENDIF
C
C     If there is a background file add .tif extension to the 
C     file name.
C
      IF (STRLEN(BAKFIL) .NE. 0) THEN
         BACKGROUND = .TRUE.
         BAKFIL = BAKFIL(1:STRLEN(BAKFIL)) // '.tif'
      ELSE
         BACKGROUND = .FALSE.
      ENDIF
C
C     Read in background file if there is one.
C
      IF (BACKGROUND) THEN
         TIFF_ERROR = OPEN_TIFF_FILE(BAKFIL, DEBUG)
         IF (TIFF_ERROR .NE. 0) GOTO 901
         TIFF_ERROR = READ_TIFF_DATA(BAKDATA, DEBUG)
         IF (TIFF_ERROR .NE. 0) GOTO 902
         BAKWIDTH  = TIFF_IMAGE_WIDTH()
         BAKHEIGHT = TIFF_IMAGE_HEIGHT()         
         CALL CLOSE_TIFF_FILE()
      ENDIF
C
C     Initialise merge array
C
      MERGED     = 0
      MERGENUM   = 0
      NPROCESSED = 0
      FRAMENUM   = START_FRAME
      DO I = 1, MAXIMSIZE
         FRAMEDATA(I) = 65536
         MERGEDATA(I) = 0
         MINPIX(I) = 65536
      ENDDO
C
C     Process files
C
      DO 10 I = START_FRAME, END_FRAME
C
C     Construct frame file name
C
         CALL ITOA(I, TIFFNUM)
         TIFFNAME = 
     $        EXPNAM(1:STRLEN(EXPNAM)) //
     $        TIFFNUM(1:STRLEN(TIFFNUM)) // '.tif'
C
C     Read TIFF file
C
         TIFF_ERROR = OPEN_TIFF_FILE(TIFFNAME, DEBUG)
         IF (TIFF_ERROR .NE. 0) THEN
            IF (TIFF_ERROR .EQ. 1) THEN
               WRITE(*,*) 'Warning: ',TIFFNAME(1:STRLEN(TIFFNAME)),
     $              ' does not exist. Skipping file.'
               GOTO 10
            ELSE
               GOTO 903
            ENDIF
         ENDIF
C
C     If doing the cosmic ray correction then read into
C     raw data array else read directly into the frame
C     data array.
C
         IF (COSMIC .GT. 0) THEN
            TIFF_ERROR = READ_TIFF_DATA(DATA, DEBUG)
         ELSE
            TIFF_ERROR = READ_TIFF_DATA(FRAMEDATA, DEBUG)
         ENDIF
         IF (TIFF_ERROR .NE. 0) GOTO 904
         FRAMEWIDTH  = TIFF_IMAGE_WIDTH()
         FRAMEHEIGHT = TIFF_IMAGE_HEIGHT()
         IF (BACKGROUND .AND. (FRAMEWIDTH .NE. BAKWIDTH .OR.
     $        FRAMEHEIGHT .NE. BAKHEIGHT)) GOTO 905
         CALL CLOSE_TIFF_FILE()
C
         IF (COSMIC .GT. 0) THEN
            DO J = 1, (FRAMEWIDTH*FRAMEHEIGHT)
               IF (DATA(J) .LT. FRAMEDATA(J)) FRAMEDATA(J) = DATA(J)
            ENDDO
            NPROCESSED = NPROCESSED + 1
            IF (NPROCESSED .EQ. COSMIC) THEN
C
               CALL ITOA(FRAMENUM, FLFNUM)
               FLFNAME = 
     $              EXPNAM(1:STRLEN(EXPNAM)) // 'c' //
     $              FLFNUM(1:STRLEN(FLFNUM)) // '.flf'
               WRITE(*,*) 'Writing cosmic-ray corrected frame: ', 
     $              FLFNAME(1:STRLEN(FLFNAME))
               NSATPIX = WRITE_FLF_FILE(FLFNAME(1:STRLEN(FLFNAME)),
     $              FRAMEDATA, FRAMEWIDTH, FRAMEHEIGHT)
C
               CALL PROCESS_FRAME(FRAMENUM)
               FRAMENUM = FRAMENUM + 1
               NPROCESSED = 0
            ENDIF
         ELSE
            FRAMENUM = I
            CALL PROCESS_FRAME(FRAMENUM)
         ENDIF
C     
 10   CONTINUE
C

      IF (DYNBAK) THEN
C
C     Original frame set has been written. Write the frame of
C     minimum pixels then re-read outputed FLF files and subtract
C     the minimum pixel frame from them.
C
         IF (NMERGE .GT. 0) THEN
            FLFNAME = EXPNAM(1:STRLEN(EXPNAM)) // 'm_dyav.flf'
         ELSE
            FLFNAME = EXPNAM(1:STRLEN(EXPNAM)) // '_dyav.flf'
         ENDIF
         WRITE(*,*) 'Writing minimum frame: ', 
     $        FLFNAME(1:STRLEN(FLFNAME))
         NSATPIX = WRITE_FLF_FILE(FLFNAME(1:STRLEN(FLFNAME)),
     $        MINPIX, FRAMEWIDTH, FRAMEHEIGHT)
         CALL SUB_DYNBAK(MINPIX, FRAMEWIDTH, FRAMEHEIGHT)
      ENDIF
C      
C
C     Warn about any unmerged files.
C
      IF (MERGED .GT. 0) THEN
         WRITE(*,*) ''
         WRITE(*,*) 
     $        'Warning: The last',MERGED,' file(s) were not merged!'
         WRITE(*,*) 'The number of input files was not a multiple'
         WRITE(*,*) 'of the merge number in the -m option.'
      ENDIF
      RETURN
C
C     Error traps
C
 901  WRITE(*,*) '** Error opening background file: ', TIFF_ERROR
      RETURN
 902  WRITE(*,*) '** Error reading background data: ', TIFF_ERROR
      RETURN
 903  WRITE(*,*) '** Error opening TIFF file: ', TIFFNAME
      RETURN            
 904  WRITE(*,*) '** Error reading data for frame: ', TIFFNAME
      RETURN
 905  WRITE(*,*) '** Error: frame dimensions not the same'
      WRITE(*,*) '** as background image dimensions:', TIFFNAME
      RETURN
      END
C
C
C
C**********************************************************************
C     Routine: PROCESS_FRAME
C
C     Description:
C        Subtracts the dynamic background calculated from a frame
C        of pixel minimums from the merged frames written to disc.
C**********************************************************************
      SUBROUTINE PROCESS_FRAME(FRAMENUM)
      IMPLICIT NONE
      INCLUDE 'cmdln.inc'
      INCLUDE 'flf.inc'         ! flf.inc must come before jitax77.inc
      INCLUDE 'jitax77.inc'
C
C     Parameters
C
      INTEGER FRAMENUM          ! Current frame number
C
C     Functions
C
      INTEGER WRITE_FLF_FILE
      INTEGER STRLEN
C
C     Variables
C
      INTEGER NSATPIX           ! Number of saturated pixels
      REAL    AVE               ! Average pixel value
      INTEGER I                 ! Loop counter
      CHARACTER*88 FLFNAME      ! The current FLF file name
      CHARACTER*4  FLFNUM       ! The current FLF file number
C
      FLFNAME = ''
      FLFNUM = ''
C
C     Read data into merge array.
C
      DO I = 1, (FRAMEWIDTH*FRAMEHEIGHT)
         IF (BACKGROUND) THEN
            MERGEDATA(I) = MERGEDATA(I)+FRAMEDATA(I)-BAKDATA(I)
         ELSE
            MERGEDATA(I) = MERGEDATA(I)+FRAMEDATA(I)
         ENDIF
         IF (MERGEDATA(I) .LT. 0) MERGEDATA(I) = 0
      ENDDO
      MERGED = MERGED + 1
C
C     If we have merged the required number of frames
C     then write the merged frame to an FLF file
C
      IF (MERGED .GE. NMERGE) THEN
         DO I = 1, (FRAMEWIDTH*FRAMEHEIGHT)
            IF (NMERGE .GT. 1 .AND. AVERAGE) THEN
               AVE = REAL(MERGEDATA(I))/NMERGE
               MERGEDATA(I) = NINT(AVE)
            ENDIF
         ENDDO
C     
         IF (NMERGE .GT. 0) THEN
C
C     -m # options specified so output merged files and keep the
C     file number consecutive even if an input file is missing
C     
            CALL ITOA(MERGENUM+START_FRAME, FLFNUM)
            FLFNAME = 
     $           EXPNAM(1:STRLEN(EXPNAM)) // 'm' //
     $           FLFNUM(1:STRLEN(FLFNUM)) // '.flf'
            WRITE(*,*) 'Writing merged frame: ', 
     $           FLFNAME(1:STRLEN(FLFNAME))
         ELSE
C     
C     No -m # option provided (NMERGE = 0). Output FLF files that
C     correspond to the input filenames even if there are missing
C     input files. Any gaps in the input file list will be reproduced
C     in the output list.
C     
            CALL ITOA(FRAMENUM, FLFNUM)
            FLFNAME = 
     $           EXPNAM(1:STRLEN(EXPNAM)) //
     $           FLFNUM(1:STRLEN(FLFNUM)) // '.flf'
            WRITE(*,*) 'Writing frame: ', 
     $           FLFNAME(1:STRLEN(FLFNAME))
         ENDIF
C     
C     From all merge frames pick the minimum pixel values.
C     
         IF (DYNBAK) THEN
            DO I = 1, (FRAMEWIDTH*FRAMEHEIGHT)
               IF(MERGEDATA(I) .LT. MINPIX(I)) THEN
                  MINPIX(I) = MERGEDATA(I)
               ENDIF
            ENDDO
         ENDIF
         NSATPIX = WRITE_FLF_FILE(FLFNAME(1:STRLEN(FLFNAME)),
     $        MERGEDATA, FRAMEWIDTH, FRAMEHEIGHT)
C     
C     Warn if there are saturated pixels in last file.
C     
         IF (NSATPIX .GT. 0) THEN
            WRITE(*,*) 
     $           'Warning there were',NSATPIX,' saturated pixels'
     $           // ' in the last FLF file!'
         ENDIF
C     
         MERGENUM = MERGENUM + 1
         MERGED = 0
         DO I = 1, MAXIMSIZE
            MERGEDATA(I) = 0
         ENDDO
      ENDIF
      END
C     
C
C
C**********************************************************************
C     Routine: SUB_DYNBAK
C
C     Description:
C        Subtracts the dynamic background calculated from a frame
C        of pixel minimums from the merged frames written to disc.
C**********************************************************************
      SUBROUTINE SUB_DYNBAK(DYNBKDAT, WIDTH, HEIGHT)
      IMPLICIT NONE
      INCLUDE 'cmdln.inc'
      INCLUDE 'flf.inc'
C
C     Parameters
C     
      INTEGER DYNBKDAT(*)       ! Dynamic background data
      INTEGER WIDTH
      INTEGER HEIGHT
C
C     Functions
C
      INTEGER STRLEN
      INTEGER READ_FLF_FILE
      INTEGER WRITE_FLF_FILE
C
C     Variables
C
      INTEGER I, J                  ! Loop counters
      INTEGER MERGED                ! The number of frames merged
      INTEGER MERGENUM              ! Number of files merged so far
      INTEGER DATA(FLFSIZE)         ! The FLF data.
      INTEGER NSATPIX               ! Number of saturated pixels
      CHARACTER*88 FLFNAME          ! The current FLF file name
      CHARACTER*4  FLFNUM           ! The current FLF file number
      INTEGER      ERROR
C
      FLFNAME = ''
      FLFNUM = ''
      MERGED = 0
      MERGENUM = 0
      DO 20 I = START_FRAME, END_FRAME
         MERGED = MERGED + 1
         IF (MERGED .GE. NMERGE) THEN
            IF (NMERGE .GT. 0) THEN
               CALL ITOA(MERGENUM+START_FRAME, FLFNUM)
               FLFNAME = 
     $              EXPNAM(1:STRLEN(EXPNAM)) // 'm' //
     $              FLFNUM(1:STRLEN(FLFNUM)) // '.flf'
            ELSE
               CALL ITOA(I, FLFNUM)
               FLFNAME = 
     $              EXPNAM(1:STRLEN(EXPNAM)) //
     $              FLFNUM(1:STRLEN(FLFNUM)) // '.flf'
            ENDIF
            ERROR = READ_FLF_FILE(FLFNAME, DATA, WIDTH, HEIGHT)
            IF (ERROR .NE. 0) GOTO 20
C
C     Subtract the dynamic background frame
C
            DO J = 1, (WIDTH*HEIGHT)
               DATA(J) = DATA(J) - DYNBKDAT(J)
            ENDDO
C
            IF (NMERGE .GT. 0) THEN
               CALL ITOA(MERGENUM+START_FRAME, FLFNUM)
               FLFNAME = 
     $              EXPNAM(1:STRLEN(EXPNAM)) // 'md' //
     $              FLFNUM(1:STRLEN(FLFNUM)) // '.flf'
            ELSE
               CALL ITOA(I, FLFNUM)
               FLFNAME = 
     $              EXPNAM(1:STRLEN(EXPNAM)) // 'd' //
     $              FLFNUM(1:STRLEN(FLFNUM)) // '.flf'
            ENDIF
            WRITE(*,*) 'Writing: ', FLFNAME(1:STRLEN(FLFNAME))
            NSATPIX = WRITE_FLF_FILE(FLFNAME(1:STRLEN(FLFNAME)),
     $           DATA, WIDTH, HEIGHT)
C
            MERGENUM = MERGENUM + 1
            MERGED = 0
         ENDIF
 20      CONTINUE
C
      END
C
C
C
C**********************************************************************
C     Routine: ITOA
C
C     Description:
C        Converts an integer into an ASCII string.
C**********************************************************************
      SUBROUTINE ITOA(NUM, STRING)
      IMPLICIT NONE
C
      INTEGER NUM
      CHARACTER*(*) STRING
C
      INTEGER DIGITS
      INTEGER TMPNUM
      INTEGER I
C
C     Count the number of digits
C
      DIGITS = 0
      TMPNUM = NUM
 10   TMPNUM = TMPNUM/10
      DIGITS = DIGITS + 1
      IF (TMPNUM .GT. 0) GOTO 10
C
C     Convert digits into a character string
C
      TMPNUM = NUM
      DO I = DIGITS, 1, -1
         STRING(I:I) = CHAR(MOD(TMPNUM, 10)+48)
         TMPNUM = TMPNUM/10
      ENDDO
      END
C
